@@ -62,7 +62,9 @@ namespace massXpert
       SIMPLIFY_FORMULA
     };
   
-  CalculatorWnd::CalculatorWnd(QString &filePath)
+  CalculatorWnd::CalculatorWnd(QString &filePath, 
+                               const QString &mono, 
+                               const QString &avg)
   {
     m_forciblyClose = false;
     m_noDelistWnd = false;
@@ -132,6 +134,24 @@ namespace massXpert
 
     m_ui.formulaComboBox->lineEdit()->clear();
 
+    // If the strings mono and avg are non null, then set their values
+    // to the seed masses.
+
+    if(!mono.isNull())
+      {
+        m_ui.seedMonoMassLineEdit->setText(mono);
+
+        // qDebug() << __FILE__ << __LINE__ 
+        //          << "Set the mono mass to" << mono;
+      }
+    
+    if(!avg.isNull())
+      {
+        m_ui.seedAvgMassLineEdit->setText(avg);
+
+        // qDebug() << __FILE__ << __LINE__ 
+        //          << "Set the avg mass to" << avg;
+      }
     
     show();
   }
@@ -336,8 +356,7 @@ namespace massXpert
     
     m_ui.formulaActionsComboBox->insertItem(SIMPLIFY_FORMULA,
 					    tr("Simplify formula"));
-    
-
+        
     if (!setupChemicalPad())
       return false;
     
@@ -1187,8 +1206,6 @@ namespace massXpert
   int
   CalculatorWnd::accountFormula(const QString &formula, int times) 
   {
-    Application *application = static_cast<Application *>(qApp);
-    QLocale locale = application->locale();
     QString text;
     int count;
   
@@ -1267,8 +1284,6 @@ namespace massXpert
   int
   CalculatorWnd::accountMonomer()
   {
-    Application *application = static_cast<Application *>(qApp);
-    QLocale locale = application->locale();
     QString text;
   
     // Get to see if there is a necessity to account for the displayed
@@ -1316,8 +1331,6 @@ namespace massXpert
   int
   CalculatorWnd::accountModif ()
   {
-    Application *application = static_cast<Application *>(qApp);
-    QLocale locale = application->locale();
     QString text;
 
     // Get to see if there is a necessity to account for the displayed
@@ -1364,8 +1377,6 @@ namespace massXpert
   int
   CalculatorWnd::accountSequence()
   {
-    Application *application = static_cast<Application *>(qApp);
-    QLocale locale = application->locale();
     QString text;
 
     // Get to see if there is a necessity to account for the displayed
@@ -1517,9 +1528,9 @@ namespace massXpert
   {
     m_ui.formulaComboBox->clear();
   }
+
   
-  
-  void
+  QString
   CalculatorWnd::simplifyFormula()
   {
     // Imagine we do lots of formula stuff, and we want to refactor a
@@ -1538,13 +1549,13 @@ namespace massXpert
 			     tr("massXpert - Calculator"),
 			     tr("No atom list is available"),
 			     QMessageBox::Ok);
-	return;
+	return QString("");;
       }
   
     QString text = m_ui.formulaComboBox->currentText();
     
     if (text.isEmpty())
-      return;
+      return QString("");
     
     // We want to remove spaces from the text. This is useful so the
     // user might enter spaces in the formula to ease re-reading the
@@ -1568,10 +1579,10 @@ namespace massXpert
 			     tr("Failed to parse the formula."),
 			     QMessageBox::Ok);
 	
-	return;
+	return QString("");
       }
     
-    QString simplified = fake.simplify();
+    QString simplified = fake.elementalComposition();
 
     //    qDebug() << "simplified:" << simplified;
 
@@ -1586,8 +1597,7 @@ namespace massXpert
 	m_ui.formulaComboBox->lineEdit()->setText(simplified);
       }
     
-    return;
-  }
-  
+    return simplified;
+  }  
 
 } // namespace massXpert
