@@ -45,15 +45,14 @@
 #include "cleavageDlg.hpp"
 #include "application.hpp"
 
-
 namespace massXpert
 {
 
   CleavageDlg::CleavageDlg(QWidget *parent,
-			    Polymer *polymer,
-			    const PolChemDef *polChemDef,
-			    const CalcOptions &calcOptions,
-			    const IonizeRule *ionizeRule)
+                           Polymer *polymer,
+                           const PolChemDef *polChemDef,
+                           const CalcOptions &calcOptions,
+                           const IonizeRule *ionizeRule)
     : QDialog(parent),
       mp_polymer(polymer), 
       mp_polChemDef(polChemDef), 
@@ -70,20 +69,15 @@ namespace massXpert
     populateCleaveSpecListWidget();
     populateSelectedOligomerData();
     
-    setupTreeView();
-    
-    m_ui.leftCapPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
-    m_ui.rightCapPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
-
-    m_ui.leftModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
-    m_ui.rightModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
-
-    m_ui.modifsPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
-
-    m_ui.crossLinksPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
-
+    // Set pointers to 0 so that after the setupTableView call below
+    // they'll get their proper value. We'll then be able to free all
+    // that stuff in the destructor.
     mpa_cleaver = 0;
-  
+    mpa_proxyModel = 0;
+    mpa_oligomerTableViewModel = 0;
+    
+    setupTableView();
+    
     // The tolerance when filtering mono/avg masses...
     QStringList stringList;
   
@@ -184,12 +178,16 @@ namespace massXpert
 
   CleavageDlg::~CleavageDlg()
   {
+    // Delete the oligomers in the list of such instances.
+
+    while(!m_oligomerList.isEmpty())
+      delete m_oligomerList.takeFirst();
+                                    
     delete mpa_resultsString;
   
-    delete mpa_oligomerTreeViewModel;
-
+    delete mpa_oligomerTableViewModel;
     delete mpa_proxyModel;
-  
+
     delete mpa_cleaver;
   }
 
@@ -345,19 +343,20 @@ bool
 
 
   void 
-  CleavageDlg::setupTreeView()
+  CleavageDlg::setupTableView()
   {
      // Model stuff all thought for sorting.
-    mpa_oligomerTreeViewModel = 
-      new CleaveOligomerTreeViewModel(&m_oligomerList, this);
+    mpa_oligomerTableViewModel = 
+      new CleaveOligomerTableViewModel(&m_oligomerList, this);
 
-    mpa_proxyModel = new CleaveOligomerTreeViewSortProxyModel(this);
-    mpa_proxyModel->setSourceModel(mpa_oligomerTreeViewModel);
+    mpa_proxyModel = new CleaveOligomerTableViewSortProxyModel(this);
+    mpa_proxyModel->setSourceModel(mpa_oligomerTableViewModel);
     mpa_proxyModel->setFilterKeyColumn(-1);
-  
-    m_ui.oligomerTreeView->setModel(mpa_proxyModel);
-    m_ui.oligomerTreeView->setParentDlg(this);
-    mpa_oligomerTreeViewModel->setTreeView(m_ui.oligomerTreeView);
+
+    m_ui.oligomerTableView->setModel(mpa_proxyModel);
+    m_ui.oligomerTableView->setParentDlg(this);
+    m_ui.oligomerTableView->setOligomerList(&m_oligomerList);
+    mpa_oligomerTableViewModel->setTableView(m_ui.oligomerTableView);
   }
   
 
@@ -371,30 +370,35 @@ bool
   void 
   CleavageDlg::cleave()
   {
-    // First update the selection data from the sequence editor window.
-    if (!populateSelectedOligomerData())
-      return;
-    
     // What's the cleavage agent ?
     int value = m_ui.cleavageAgentListWidget->currentRow();
-
     CleaveSpec *cleaveSpec = mp_polChemDef->cleaveSpecList().at(value);
     
+    // What's the asked level of partial cleavages?
     value = m_ui.partialCleavageSpinBox->value();
   
     CleaveOptions cleaveOptions(*cleaveSpec, value, false);
-  
-    // Set the ionization levels.
+
+    // Set the ionization levels to the local cleavage options object.
     int valueStart = m_ui.ionizeLevelStartSpinBox->value();
     int valueEnd = m_ui.ionizeLevelEndSpinBox->value();
 
     cleaveOptions.setIonizeLevels(valueStart, valueEnd);
     
+    // The list in which we'll store all the allocated oligomers.
     OligomerList oligomerList;
     
     delete mpa_cleaver;
     mpa_cleaver = 0;
-    
+
+    // Update the mass calculation engine's configuration data
+    // directly from the sequence editor window.
+    m_calcOptions = mp_editorWnd->calcOptions();
+        
+    // Update the selection data from the sequence editor window.
+    if (!populateSelectedOligomerData())
+      return;
+
     mpa_cleaver= new Cleaver(mp_polymer, mp_polChemDef, cleaveOptions,
 			     m_calcOptions, *mp_ionizeRule);
 
@@ -402,95 +406,110 @@ bool
     
     if (!mpa_cleaver->cleave())
       {
+        delete mpa_cleaver;
+        
 	QMessageBox::critical(this, 
-			       tr("massXpert - Polymer Cleavage"),
-			       tr("Failed to perform cleavage."),
-			       QMessageBox::Ok);
+                              tr("massXpert - Polymer Cleavage"),
+                              tr("Failed to perform cleavage."),
+                              QMessageBox::Ok);
 	return;
       }
-        
-    // We now have new oligomers in oligomerList. We either stack
-    // these on top of the previous ones, or we replace the previous
-    // ones with the new ones.
-    // Are we stacking new oligomer on top of the old ones?
+
+    // At this point we have a brand new set of oligomers in the local
+    // list of oligomers (oligomerList). We either stack these on top
+    // of the previous ones, or we replace the previous ones with the
+    // new ones.  Are we stacking new oligomers on top of the old
+    // ones?
 
     if (!m_ui.stackOligomersCheckBox->isChecked())
-      mpa_oligomerTreeViewModel->removeOligomers();
-    
-    
+      {
+        // We are going to remove *all* the previous oligomers.
+        mpa_oligomerTableViewModel->removeOligomers(0, -1);
+      }
+        
     // At this point we can set up the data to the treeview model.
+
     int initialOligomers = oligomerList.size();
-    int addedOligomers = mpa_oligomerTreeViewModel->addOligomers(&oligomerList);
+
+    // qDebug() << __FILE__ << __LINE__
+    //          << "initialOligomers:" << initialOligomers;
+ 
+    // We are *transferring* the oligomers from oligomerList to the
+    // list of oligomers that is in the model : we are not duplicating
+    // the oligomers. When the transfer has been done, oligomerList
+    // will be empty.
+    int addedOligomers = 
+      mpa_oligomerTableViewModel->addOligomers(oligomerList);
+
+    // qDebug() << __FILE__ << __LINE__
+    //          << "addedOligomers:" << addedOligomers;
     
     if (initialOligomers != addedOligomers)
-      qFatal("Fatal error at %s@%d. Program aborted.",
-	      __FILE__, __LINE__);
-
-    if (oligomerList.size())
-      qFatal("Fatal error at %s@%d. Program aborted.",
-	      __FILE__, __LINE__);
-        
-    // Logically, the oligomerList should be empty by now.
-    Q_ASSERT(oligomerList.isEmpty());
+      qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+    
+    // Now that we have *transferred* (not copied) all the oligomers
+    // in the model, we can clear the oligomerList without freeing the
+    // instances it currently contain...
+    oligomerList.clear();
     
-    updateCleavageDetails();
-
     // Set focus to the treeView.
-    m_ui.oligomerTreeView->setFocus();
+    m_ui.oligomerTableView->setFocus();
     
     QString title;
     
-    int oligomerCount = mpa_oligomerTreeViewModel->rowCount();
+    int oligomerCount = mpa_oligomerTableViewModel->rowCount();
     
     Application *application = static_cast<Application *>(qApp);
     QLocale locale = application->locale();
 
     if (!oligomerCount)
-      title = tr("Oligomers(empty list)");
+      title = tr("Oligomers (empty list)");
     else if (oligomerCount == 1)
-      title = tr("Oligomers(one item)");
+      title = tr("Oligomers (one item)");
     else
-      title = tr("Oligomers(%1 items)")
+      title = tr("Oligomers (%1 items)")
 	.arg(locale.toString(oligomerCount));
 	
     m_ui.oligomerGroupBox->setTitle(title);
-  }
+
+    // Update the cleavage details so that we know how the oligos were
+    // computed.
+    updateCleavageDetails(m_calcOptions);
+}
 
 
   void
-  CleavageDlg::updateCleavageDetails()
+  CleavageDlg::updateCleavageDetails(const CalcOptions &calcOptions)
   {
-    if (m_calcOptions.capping() & MXT_CAP_LEFT)
-      m_ui.leftCapPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.capping() & MXT_CAP_LEFT)
+      m_ui.leftCapCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.leftCapPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.leftCapCheckBox->setCheckState(Qt::Unchecked);
 
-    if (m_calcOptions.capping() & MXT_CAP_RIGHT)
-      m_ui.rightCapPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.capping() & MXT_CAP_RIGHT)
+      m_ui.rightCapCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.rightCapPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.rightCapCheckBox->setCheckState(Qt::Unchecked);
 
-
-    if (m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_MODIF)
-      m_ui.modifsPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_LEFT_END_MODIF)
+      m_ui.leftModifCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.modifsPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.leftModifCheckBox->setCheckState(Qt::Unchecked);
 
-    if (m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_CROSS_LINK)
-      m_ui.crossLinksPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_RIGHT_END_MODIF)
+      m_ui.rightModifCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.crossLinksPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.rightModifCheckBox->setCheckState(Qt::Unchecked);
 
-
-    if (m_calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_LEFT_END_MODIF)
-      m_ui.leftModifPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_MODIF)
+      m_ui.modificationsCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.leftModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.modificationsCheckBox->setCheckState(Qt::Unchecked);
 
-    if (m_calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_RIGHT_END_MODIF)
-      m_ui.rightModifPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_CROSS_LINK)
+      m_ui.crossLinksCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.rightModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.crossLinksCheckBox->setCheckState(Qt::Unchecked);
   }
 
 
@@ -743,6 +762,7 @@ bool
   }
 
 
+  
 
   // The results-exporting functions. ////////////////////////////////
   // The results-exporting functions. ////////////////////////////////
@@ -757,7 +777,7 @@ bool
 
     if (index == 0)
       {
-	exportResultsClipboard();
+	exportResultsToClipboard();
       }
     else if (index == 1)
       {
@@ -785,7 +805,7 @@ bool
     bool withSequence = m_ui.withSequenceCheckBox->isChecked();
     
     QString *text = 
-      m_ui.oligomerTreeView->selectedOligomersAsPlainText(withSequence);
+      m_ui.oligomerTableView->selectedOligomersAsPlainText(withSequence);
 
     *mpa_resultsString += *text;
     
@@ -794,7 +814,7 @@ bool
 
 
   bool 
-  CleavageDlg::exportResultsClipboard()
+  CleavageDlg::exportResultsToClipboard()
   {
     prepareResultsTxtString();
   
