@@ -48,16 +48,16 @@
 #include "ui_cleavageDlg.h"
 #include "cleaver.hpp"
 #include "sequenceEditorWnd.hpp"
-#include "cleaveOligomerTreeViewModel.hpp"
-#include "cleaveOligomerTreeViewSortProxyModel.hpp"
+#include "cleaveOligomerTableViewModel.hpp"
+#include "cleaveOligomerTableViewSortProxyModel.hpp"
 
 
 namespace massXpert
 {
 
-  class CleaveOligomerTreeViewModel;
-  class CleaveOligomerTreeViewSortProxyModel;
-
+  class CleaveOligomerTableViewModel;
+  class CleaveOligomerTableViewSortProxyModel;
+  
   class CleavageDlg : public QDialog
   {
     Q_OBJECT
@@ -80,8 +80,8 @@ namespace massXpert
 
     Cleaver *mpa_cleaver;
   
-    CleaveOligomerTreeViewModel *mpa_oligomerTreeViewModel;
-    CleaveOligomerTreeViewSortProxyModel *mpa_proxyModel;
+    CleaveOligomerTableViewModel *mpa_oligomerTableViewModel;
+    CleaveOligomerTableViewSortProxyModel *mpa_proxyModel;
 
     void closeEvent(QCloseEvent *event);
 
@@ -90,7 +90,7 @@ namespace massXpert
     double m_tolerance;
     QWidget *mp_focusWidget;
 
-    void setupTreeView();
+    void setupTableView();
       
   public:
     CleavageDlg(QWidget *,
@@ -104,14 +104,14 @@ namespace massXpert
 
     SequenceEditorWnd *editorWnd();
     
-    void updateCleavageDetails();
+    void updateCleavageDetails(const CalcOptions &);
     void updateOligomerSequence(QString *);
 
     bool calculateTolerance(double);
   
     // The results-exporting functions. ////////////////////////////////
     void prepareResultsTxtString();
-    bool exportResultsClipboard();
+    bool exportResultsToClipboard();
     bool exportResultsFile();
     bool selectResultsFile();
     //////////////////////////////////// The results-exporting functions.
