@@ -60,8 +60,7 @@ namespace massXpert
       mp_ionizeRule(ionizeRule)
   {
     Q_ASSERT(parent);
-    Q_ASSERT(!mp_polymer.isNull() && mp_polChemDef);
-    Q_ASSERT(mp_ionizeRule);
+    Q_ASSERT(!mp_polymer.isNull() && mp_polChemDef && mp_ionizeRule);
     
     m_ui.setupUi(this);
   
@@ -73,15 +72,18 @@ namespace massXpert
     m_ui.fragmentationPatternListWidget->
       setSelectionMode(QAbstractItemView::MultiSelection);
 
-    mpa_oligomerTreeViewModel = 0;
+    mpa_oligomerTableViewModel = 0;
     mpa_proxyModel = 0;
   
-    // Absolutely required that the pointer be 0 at creation because we
-    // rely on its being non-0 to free it when required(pointer used
-    // many times).
+    // Set pointers to 0 so that after the setupTableView call below
+    // they'll get their proper value. We'll then be able to free all
+    // that stuff in the destructor.
     mpa_fragmenter = 0;
-  
-
+    mpa_proxyModel = 0;
+    mpa_oligomerTableViewModel = 0;
+    
+    setupTableView();
+    
     // The tolerance when filtering mono/avg masses...
     QStringList stringList;
   
@@ -183,34 +185,17 @@ namespace massXpert
 
   FragmentationDlg::~FragmentationDlg()
   {
-    delete mpa_oligomerTreeViewModel;
-    delete mpa_proxyModel;
-    delete mpa_fragmenter;
+    // Delete the oligomers in the list of such instances.
+
+    while(!m_oligomerList.isEmpty())
+      delete m_oligomerList.takeFirst();
+
     delete mpa_resultsString;
-  }
 
+    delete mpa_oligomerTableViewModel;
+    delete mpa_proxyModel;
 
-  void 
-  FragmentationDlg::closeEvent(QCloseEvent *event)
-  {
-    if (event)
-      printf("%s", "");
-  
-    QSettings settings 
-     (static_cast<Application *>(qApp)->configSettingsFilePath(), 
-       QSettings::IniFormat);
-  
-    settings.beginGroup("fragmentation_dlg");
-
-    settings.setValue("geometry", saveGeometry());
-
-    settings.setValue("oligomersSplitter", 
-		       m_ui.oligomersSplitter->saveState());
-
-    settings.setValue("oligoDetailsSplitter", 
-		       m_ui.oligoDetailsSplitter->saveState());
-
-    settings.endGroup();
+    delete mpa_fragmenter;
   }
 
 
@@ -271,6 +256,24 @@ namespace massXpert
   }
 
 
+  void 
+  FragmentationDlg::setupTableView()
+  {
+     // Model stuff all thought for sorting.
+    mpa_oligomerTableViewModel = 
+      new FragmentOligomerTableViewModel(&m_oligomerList, this);
+
+    mpa_proxyModel = new FragmentOligomerTableViewSortProxyModel(this);
+    mpa_proxyModel->setSourceModel(mpa_oligomerTableViewModel);
+    mpa_proxyModel->setFilterKeyColumn(-1);
+
+    m_ui.oligomerTableView->setModel(mpa_proxyModel);
+    m_ui.oligomerTableView->setParentDlg(this);
+    m_ui.oligomerTableView->setOligomerList(&m_oligomerList);
+    mpa_oligomerTableViewModel->setTableView(m_ui.oligomerTableView);
+  }
+
+
   SequenceEditorWnd *
   FragmentationDlg::editorWnd()
   {
@@ -298,9 +301,9 @@ namespace massXpert
     QList<QListWidgetItem *> selectionList = 
       m_ui.fragmentationPatternListWidget->selectedItems();
   
-    if (!selectionList.size())
+    if (selectionList.isEmpty())
       return;
-
+    
     // We'll need to use the specifications in the polymer chemistry
     // definition... for each frag options object.
     QList<FragSpec *> refList = mp_polChemDef->fragSpecList();
@@ -338,8 +341,46 @@ namespace massXpert
 	      }
       
 	    FragOptions *fragOptions = new FragOptions(fragSpec);
-	  
-	    // We got the coordinates of the oligomer to fragment in the
+
+            // The user might ask that some formulas be applied while
+            // making the fragmentation with the selection
+            // fragmentation option. Lets feed the FragOptions
+            // instance with the related formulas.
+
+            if(m_ui.decompositionsMinusWaterCheckBox->checkState() == 
+                Qt::Checked)
+              {
+                fragOptions->addFormula(QString("-H2O"));
+              }
+            
+            if(m_ui.decompositionsMinusAmmoniaCheckBox->checkState() == 
+                Qt::Checked)
+              {
+                fragOptions->addFormula(QString("-NH3"));
+              }
+
+            if(!m_ui.decompositionsFormulaLineEdit->text().isEmpty())
+              {
+                Formula formula(m_ui.decompositionsFormulaLineEdit->text());
+
+                // At this point, we should check if the formula is valid.
+                const QList<Atom *> &atomRefList = mp_polChemDef->atomList();
+
+                if(!formula.validate(atomRefList))
+                  {
+                    QMessageBox::information
+                      (0, 
+                       tr("massXpert - Fragmentation"),
+                       tr("Failed to validate the decomposition formula."),
+                       QMessageBox::Ok);
+
+                    return;
+                  }
+                
+                fragOptions->addFormula(formula);
+              }
+            
+            // We got the coordinates of the oligomer to fragment in the
 	    // mp_calcOptions from the sequence editor window.
 
 	    Coordinates *coordinates = m_calcOptions.coordinateList().last();
@@ -379,90 +420,120 @@ namespace massXpert
     // widget.
 
     Q_ASSERT(fragOptionList.size() == selectionList.size());
-  
+
+    // The list in which we'll store all the allocated oligomers.
+    OligomerList oligomerList;
+
     // Now that we have all the fragmentation options in the list, we
     // can allocate the fragmenter! Note that this function might be
-    // called many time, which means that we first have to free the
+    // called many times, which means that we first have to free the
     // mpa_fragmenter allocated member if non-0.
 
     delete mpa_fragmenter;
     mpa_fragmenter = 0;
     
+    // Update the mass calculation engine's configuration data
+    // directly from the sequence editor window.
+    m_calcOptions = mp_editorWnd->calcOptions();
+
     mpa_fragmenter = new Fragmenter(mp_polymer, mp_polChemDef,
 				     fragOptionList, 
 				     m_calcOptions, *mp_ionizeRule);
   
+    mpa_fragmenter->setOligomerList(&oligomerList);
+
     if (!mpa_fragmenter->fragment())
       {
 	delete mpa_fragmenter;
       
-	QMessageBox msgBox;
-
-	msgBox.setText("Failed to perform the fragmentation.");
-
-	msgBox.setStandardButtons(QMessageBox::Ok);
-	
-	msgBox.exec();
-      
+	QMessageBox::critical(this, 
+                              tr("massXpert - Polymer Cleavage"),
+                              tr("Failed to perform fragmentation."),
+                              QMessageBox::Ok);
+        
 	return;
       }
   
-    QList<OligomerList *> &dataList = mpa_fragmenter->oligomerLists();
-  
-    delete mpa_oligomerTreeViewModel;
-    mpa_oligomerTreeViewModel = 0;
+    // At this point we have a brand new set of oligomers in the local
+    // list of oligomers (oligomerList). We either stack these on top
+    // of the previous ones, or we replace the previous ones with the
+    // new ones.  Are we stacking new oligomers on top of the old
+    // ones?
+
+    if (!m_ui.stackOligomersCheckBox->isChecked())
+      {
+        // We are going to remove *all* the previous oligomers.
+        mpa_oligomerTableViewModel->removeOligomers(0, -1);
+      }
+        
+    // At this point we can set up the data to the treeview model.
+
+    int initialOligomers = oligomerList.size();
+
+    // qDebug() << __FILE__ << __LINE__
+    //          << "initialOligomers:" << initialOligomers;
+ 
+    // We are *transferring* the oligomers from oligomerList to the
+    // list of oligomers that is in the model : we are not duplicating
+    // the oligomers. When the transfer has been done, oligomerList
+    // will be empty.
+    int addedOligomers = 
+      mpa_oligomerTableViewModel->addOligomers(oligomerList);
+
+    // qDebug() << __FILE__ << __LINE__
+    //          << "addedOligomers:" << addedOligomers;
     
-    delete mpa_proxyModel;
-    mpa_proxyModel = 0;
+    if (initialOligomers != addedOligomers)
+      qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
+    
+    // Now that we have *transferred* (not copied) all the oligomers
+    // in the model, we can clear the oligomerList without freeing the
+    // instances it currently contain...
+    oligomerList.clear();
     
-    // Model stuff all thought for sorting.
-    mpa_oligomerTreeViewModel = 
-      new FragmentOligomerTreeViewModel(&dataList, this);
-
-    mpa_proxyModel = new FragmentOligomerTreeViewSortProxyModel(this);
-    mpa_proxyModel->setSourceModel(mpa_oligomerTreeViewModel);
-    mpa_proxyModel->setFilterKeyColumn(-1);
-
-    m_ui.oligomerTreeView->setModel(mpa_proxyModel);
-
-    m_ui.oligomerTreeView->setParentDlg(this);
-  
-    mpa_oligomerTreeViewModel->setTreeView(m_ui.oligomerTreeView);
-  
-    updateFragmentationDetails();
-
     // Set focus to the treeView.
-    m_ui.oligomerTreeView->setFocus();
-
-    QPoint tvPoint = m_ui.oligomerTreeView->pos();
-    QList<int> splitList = m_ui.oligomersSplitter->sizes();
-    tvPoint.ry() += splitList.at(0);
-    QPoint destPoint(mapToGlobal(tvPoint));
+    m_ui.oligomerTableView->setFocus();
     
-    QToolTip::showText(destPoint, 
-			QString(tr("Generated %1 oligomers")
-				 .arg(mpa_oligomerTreeViewModel->rowCount())), 
-			m_ui.oligomerTreeView);
+    QString title;
+    
+    int oligomerCount = mpa_oligomerTableViewModel->rowCount();
+    
+    Application *application = static_cast<Application *>(qApp);
+    QLocale locale = application->locale();
+
+    if (!oligomerCount)
+      title = tr("Oligomers (empty list)");
+    else if (oligomerCount == 1)
+      title = tr("Oligomers (one item)");
+    else
+      title = tr("Oligomers (%1 items)")
+	.arg(locale.toString(oligomerCount));
+	
+    m_ui.oligomerGroupBox->setTitle(title);
+
+    // Update the fragmentation details so that we know how the oligos were
+    // computed.
+    updateFragmentationDetails(m_calcOptions);
   }
 
 
   void
-  FragmentationDlg::updateFragmentationDetails()
+  FragmentationDlg::updateFragmentationDetails(const CalcOptions &calcOptions)
   {
-    if (m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_MODIF)
-      m_ui.modifsPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_MODIF)
+      m_ui.modificationsCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.modifsPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.modificationsCheckBox->setCheckState(Qt::Unchecked);
 
-    if (m_calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_LEFT_END_MODIF)
-      m_ui.leftModifPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_LEFT_END_MODIF)
+      m_ui.leftModifCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.leftModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.leftModifCheckBox->setCheckState(Qt::Unchecked);
 
-    if (m_calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_RIGHT_END_MODIF)
-      m_ui.rightModifPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_RIGHT_END_MODIF)
+      m_ui.rightModifCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.rightModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.rightModifCheckBox->setCheckState(Qt::Unchecked);
   }
 
 
@@ -476,6 +547,29 @@ namespace massXpert
   }
 
 
+  void 
+  FragmentationDlg::closeEvent(QCloseEvent *event)
+  {
+    if (event)
+      printf("%s", "");
+  
+    QSettings settings 
+     (static_cast<Application *>(qApp)->configSettingsFilePath(), 
+       QSettings::IniFormat);
+  
+    settings.beginGroup("fragmentation_dlg");
+
+    settings.setValue("geometry", saveGeometry());
+
+    settings.setValue("oligomersSplitter", 
+		       m_ui.oligomersSplitter->saveState());
+
+    settings.setValue("oligoDetailsSplitter", 
+		       m_ui.oligoDetailsSplitter->saveState());
+
+    settings.endGroup();
+  }
+
 
   bool
   FragmentationDlg::calculateTolerance(double mass)
@@ -721,7 +815,7 @@ namespace massXpert
     bool withSequence = m_ui.withSequenceCheckBox->isChecked();
     
     QString *text = 
-      m_ui.oligomerTreeView->selectedOligomersAsPlainText(withSequence);
+      m_ui.oligomerTableView->selectedOligomersAsPlainText(withSequence);
 
     *mpa_resultsString += *text;
     
