@@ -44,8 +44,8 @@
 #include "mzLabWnd.hpp"
 #include "mainWindow.hpp"
 #include "application.hpp"
-#include "mzLabInputOligomerTreeViewDlg.hpp"
-#include "mzLabOutputOligomerTreeViewDlg.hpp"
+#include "mzLabInputOligomerTableViewDlg.hpp"
+#include "mzLabOutputOligomerTableViewDlg.hpp"
 #include "oligomerPair.hpp"
 #include "fragmentOligomer.hpp"
 
@@ -230,10 +230,10 @@ namespace massXpert
 	     this,
 	     SLOT(reionizePushButton()));
   
-    connect(m_ui.performMatchesPushButton,
+    connect(m_ui.performMatchWorkPushButton,
 	     SIGNAL(clicked()),
 	     this,
-	     SLOT(performMatchesPushButton()));
+	     SLOT(performMatchWorkPushButton()));
 
     connect(m_ui.newInputListPushButton,
 	     SIGNAL(clicked()),
@@ -255,6 +255,11 @@ namespace massXpert
 	     this,
 	     SLOT(deleteInputListItem()));
 
+    connect(m_ui.helpPushButton,
+	     SIGNAL(clicked()),
+	     this,
+	     SLOT(help()));
+
 
     if (!m_polChemDef.renderXmlPolChemDefFile())
       return false;
@@ -352,17 +357,16 @@ namespace massXpert
 
 
   void 
-  MzLabWnd::fillMolecularMassList(const OligomerList *oligomerList,
+  MzLabWnd::fillMolecularMassList(const OligomerList &oligomerList,
 				   QList<double> *listM, MassType &massType)
   {
-    Q_ASSERT(oligomerList);
     Q_ASSERT(listM);
 
     double mass = 0;
     
-    for (int iter = 0; iter < oligomerList->size(); ++iter)
+    for (int iter = 0; iter < oligomerList.size(); ++iter)
       {
-	Oligomer *oligomer = oligomerList->at(iter);
+	Oligomer *oligomer = oligomerList.at(iter);
 	
 	FragmentOligomer *fragOligomer =
 	  dynamic_cast<FragmentOligomer *>(oligomer);
@@ -389,7 +393,7 @@ namespace massXpert
   void
   MzLabWnd::applyFormulaPushButton()
   {
-    MzLabInputOligomerTreeViewDlg *dlg = 0;
+    MzLabInputOligomerTableViewDlg *dlg = 0;
     
     if (!inputListDlg(&dlg))
       {
@@ -440,7 +444,7 @@ namespace massXpert
   void
   MzLabWnd::applyMassPushButton()
   {
-    MzLabInputOligomerTreeViewDlg *dlg = 0;
+    MzLabInputOligomerTableViewDlg *dlg = 0;
     
     if (!inputListDlg(&dlg))
       {
@@ -496,7 +500,7 @@ namespace massXpert
   void
   MzLabWnd::applyThresholdPushButton()
   {
-    MzLabInputOligomerTreeViewDlg *dlg = 0;
+    MzLabInputOligomerTableViewDlg *dlg = 0;
     
     if (!inputListDlg(&dlg))    
       {
@@ -554,7 +558,7 @@ namespace massXpert
   void 
   MzLabWnd::incrementChargePushButton()
   {
-    MzLabInputOligomerTreeViewDlg *dlg = 0;
+    MzLabInputOligomerTableViewDlg *dlg = 0;
     
     if (!inputListDlg(&dlg))    
       {
@@ -594,7 +598,7 @@ namespace massXpert
   void
   MzLabWnd::reionizePushButton()
   {
-    MzLabInputOligomerTreeViewDlg *dlg = 0;
+    MzLabInputOligomerTableViewDlg *dlg = 0;
     
     if (!inputListDlg(&dlg))    
       {
@@ -636,7 +640,7 @@ namespace massXpert
   
   
   void
-  MzLabWnd::performMatchesPushButton()
+  MzLabWnd::performMatchWorkPushButton()
 
   {
     // We have to get the name of two lists. The first list's item
@@ -644,8 +648,8 @@ namespace massXpert
     // that the second list's items be modified to match the first
     // list's ones.
 
-    MzLabInputOligomerTreeViewDlg *dlg1 = 0;
-    MzLabInputOligomerTreeViewDlg *dlg2 = 0;
+    MzLabInputOligomerTableViewDlg *dlg1 = 0;
+    MzLabInputOligomerTableViewDlg *dlg2 = 0;
 
     if (!inputListsDlg(&dlg1, &dlg2))    
       {
@@ -707,21 +711,33 @@ namespace massXpert
     //    qDebug() << __FILE__ << __LINE__
     //             << "massType:" << massType;
 
+    // Allocate first the oligomer shell that we'll use later.
+    OligomerPair *oligomerPair = 0;
+
+    // What kind of match are we willing?
+    bool trueMatches = m_ui.trueMatchesCheckBox->isChecked();
+    // qDebug() << __FILE__ << __LINE__ 
+    //          << "trueMatches:" << trueMatches;
+    
+    bool falseMatches = m_ui.falseMatchesCheckBox->isChecked();
+    // qDebug() << __FILE__ << __LINE__ 
+    //          << "falseMatches:" << falseMatches;
+
     // Create the two lists that will hold the double values.
 
     // First list.
 
-    const OligomerList *listMz1 = dlg1->oligomerList();
+    const OligomerList &listMz1 = dlg1->oligomerList();
     QList<double> listM1;
     fillMolecularMassList(listMz1, &listM1, massType);
-    Q_ASSERT(listMz1->size() == listM1.size());
+    Q_ASSERT(listMz1.size() == listM1.size());
 
     // Second list.
 
-    const OligomerList *listMz2 = dlg2->oligomerList();
+    const OligomerList &listMz2 = dlg2->oligomerList();
     QList<double> listM2;
     fillMolecularMassList(listMz2, &listM2, massType);
-    Q_ASSERT(listMz2->size() == listM2.size());
+    Q_ASSERT(listMz2.size() == listM2.size());
 
     // Allocate the new list of OligomerPair instances into which
     // we'll append OligomerPairs instances corresponding to
@@ -732,66 +748,104 @@ namespace massXpert
 
     // At this point we have the two lists with molecular masses. We
     // can start doing the matches.
-
+        
     for (int iter = 0; iter < listM1.size(); ++iter)
       {
 	double mass1 =  listM1.at(iter);
 
-	for(int jter = 0; jter < listM2.size(); ++jter)
+        double tolerance = calculateTolerance(mass1);
+        
+        double lowerBorder = mass1 - tolerance;
+        double upperBorder = mass1 + tolerance;
+        
+        bool foundMatch = false;
+        
+        for(int jter = 0; jter < listM2.size(); ++jter)
 	  {
 	    double mass2 =  listM2.at(jter);
-	    double tolerance = calculateTolerance(mass1);
-           
-	    if (mass2 >= mass1 - tolerance &&
-		mass2 <= mass1 + tolerance)
+            
+            if (mass2 >= lowerBorder &&
+		mass2 <= upperBorder)
 	      {
-		//                qDebug() << __FILE__ << __LINE__
-		//                         << "M1 at index:" << iter 
-		//                         << "matches"
-		//                         << "M2 at index:" << jter;
-               
+                // qDebug() << __FILE__ << __LINE__
+                //          << "M1 at index:" << iter 
+                //          << "matches"
+                //          << "M2 at index:" << jter; 
+                
 		// At this point we can create a new OligomerPair.
-               
-		OligomerPair *oligomerPair = 
-		  new OligomerPair(listMz1->at(iter), listMz2->at(jter),
-				   massType,
-				   qAbs(mass2 - mass1), "NOT_SET");
-               
-		oligomerPairList->append(oligomerPair);
-               
-// 		qDebug() << __FILE__ << __LINE__
-// 			 << "Appended oligomer pair:"
-// 			 << oligomerPair->oligomer1()->name()
-// 			 << oligomerPair->oligomer2()->name()
-// 			 << "checksum:" << oligomerPair->oligomer1()->
-// 		  polymer()->checksum();
-	      }
-	  }
+                if(trueMatches)
+                  {
+                    oligomerPair =
+                      new OligomerPair(listMz1.at(iter), listMz2.at(jter),
+                                       massType,
+                                       qAbs(mass2 - mass1), 
+                                       true /* isMatching */,
+                                       "NOT_SET");
+
+                    oligomerPairList->append(oligomerPair);
+                  }
+                
+                // Whatever asked by the user (trueMatches or not), we
+                // must tell the following code that a match was
+                // found.
+                foundMatch = true;
+              }
+          }
+        
+        // Now that we have finished dealing with mass1, let's check
+        // if a match was found. If not and the non-matches are
+        // requested, store a locally allocated oligomerPair.
+
+        if(!foundMatch && falseMatches)
+          {
+            // Allocate a duplicate oligomer identical to the oligomer
+            // that had no match in the listMz2.
+            Oligomer *oligomer = new Oligomer(*listMz1.at(iter));
+
+            oligomerPair = 
+              new OligomerPair(listMz1.at(iter), oligomer,
+                               massType,
+                               0, 
+                               false /* isMatching */,
+                               "NOT_SET");
+            
+            oligomerPairList->append(oligomerPair);
+          }
       }
-   
+    // End of
+    // for (int iter = 0; iter < listM1.size(); ++iter)
+    
+    
+    // qDebug() << __FILE__ << __LINE__
+    //          << "Appended oligomer pair:"
+    //          << oligomerPair->oligomer1()->name()
+    //          << oligomerPair->oligomer2()->name()
+    //          << "checksum:" << oligomerPair->oligomer1()->
+    //   polymer()->checksum();
+    
     //    qDebug() << __FILE__ << __LINE__
     //             << "Found pairs:" << oligomerPairList->size();
-   
+    
     // Allocate the new dialog, we pass 
-
+  
     QString dialogName = tr("%1 vs %2")
-      .arg(dlg1->name())
+                .arg(dlg1->name())
       .arg(dlg2->name());
-   
+    
     // Ownership of oligomerPairList is going to be transferred to the
-    // dialog being created (note that to the dialog and not the tree
-    // view model in the dialog's tree view).
+    // dialog being created (note that to the dialog and not the table
+    // view model in the dialog's table view).
 
-    MzLabOutputOligomerTreeViewDlg *dlg =
-      new MzLabOutputOligomerTreeViewDlg(this, oligomerPairList, 
-					 massType1,
-					 dlg1, dlg2, dialogName);
+    MzLabOutputOligomerTableViewDlg *dlg =
+      new MzLabOutputOligomerTableViewDlg(this, oligomerPairList, 
+                                          massType1, dlg1, dlg2, 
+                                          dlg1->name(), dlg2->name());
     dlg->show();
   }
    
 
   bool
-  MzLabWnd::inputListDlg(MzLabInputOligomerTreeViewDlg **dlg)
+  MzLabWnd::inputListDlg(MzLabInputOligomerTableViewDlg **dlg)
   {
     QListWidgetItem *item = 0;
     
@@ -810,7 +864,7 @@ namespace massXpert
     
     QString text = item->text();
     
-    MzLabInputOligomerTreeViewDlg *dialog = findDlg(text);
+    MzLabInputOligomerTableViewDlg *dialog = findDlg(text);
     Q_ASSERT(dialog);
     
     *dlg = dialog;
@@ -820,8 +874,8 @@ namespace massXpert
   
   
  bool
- MzLabWnd::inputListsDlg(MzLabInputOligomerTreeViewDlg **dlg1,
-			  MzLabInputOligomerTreeViewDlg **dlg2)
+ MzLabWnd::inputListsDlg(MzLabInputOligomerTableViewDlg **dlg1,
+			  MzLabInputOligomerTableViewDlg **dlg2)
   {
     QListWidgetItem *item1 = 0;
     QListWidgetItem *item2 = 0;
@@ -839,7 +893,7 @@ namespace massXpert
     
     QString text = item1->text();
     
-    MzLabInputOligomerTreeViewDlg *dialog = findDlg(text);
+    MzLabInputOligomerTableViewDlg *dialog = findDlg(text);
     Q_ASSERT(dialog);
     
     *dlg1 = dialog;
@@ -857,44 +911,74 @@ namespace massXpert
   }
   
   
-  MzLabInputOligomerTreeViewDlg *
+  MzLabInputOligomerTableViewDlg *
   MzLabWnd::newInputList()
+  {
+    return newInputList(QString(), MXT_MASS_NONE, false);
+  }
+  
+  MzLabInputOligomerTableViewDlg *
+  MzLabWnd::newInputList(QString name, MassType massType, bool isFragment)
   {
     // Ask that the user give a name to the list.
-
+    
     bool ok;
+    
+    QString localName = name;
+    
+    if(localName.isEmpty())
+      {
+        // Each dialog must have a name. So ask for one.
+        localName = 
+          QInputDialog::getText(this, 
+                                tr("Give a name to the new input list"),
+                                tr("List name:"), 
+                                QLineEdit::Normal,
+                                tr("New input list name"), &ok);
+        
+        if (localName.isEmpty() || !ok)
+          return 0;
+      }
+    else
+      {
+        // The name is already set, only ask the user to confirm it.
+        
+        localName = 
+          QInputDialog::getText(this, 
+                                tr("Confirm the name of the new input list"),
+                                tr("List name:"), 
+                                QLineEdit::Normal,
+                                localName, &ok);
+        
+        if (localName.isEmpty() || !ok)
+          return 0;
+      }
+    
 
-    QString name = 
-      QInputDialog::getText(this, tr("Give a name to the new input list"),
-			     tr("List name:"), 
-			     QLineEdit::Normal,
-			     tr("New input list name"), &ok);
-    if (name.isEmpty() || !ok)
-      return 0;
 
     // Make sure the name is not already taken.
-    if (findDlg(name))
+    if (findDlg(localName))
       {
 	QMessageBox::warning(this,
-			      tr("massXpert: mz Lab"),
-			      tr("%1@%2\n"
-				  "The '%3' name is already taken.")
-			      .arg(__FILE__)
-			      .arg(__LINE__)
-			      .arg(name),
-			      QMessageBox::Ok);
+                             tr("massXpert: mz Lab"),
+                             tr("%1@%2\n"
+                                "The '%3' name is already taken.")
+                             .arg(__FILE__)
+                             .arg(__LINE__)
+                             .arg(localName),
+                             QMessageBox::Ok);
 	return 0;
       }
     
-    MzLabInputOligomerTreeViewDlg *dlg = 
-      new MzLabInputOligomerTreeViewDlg(this, name);
+    MzLabInputOligomerTableViewDlg *dlg = 
+      new MzLabInputOligomerTableViewDlg(this, localName, massType, isFragment);
     
     dlg->show();
     
     // Now we can add the item to the lists of input...
 
-    m_ui.catalogue1ListWidget->addItem(name);
-    m_ui.catalogue2ListWidget->addItem(name);
+    m_ui.catalogue1ListWidget->addItem(localName);
+    m_ui.catalogue2ListWidget->addItem(localName);
 
     // Finally we can add the list to the list of dialog windows.
     
@@ -906,12 +990,41 @@ namespace massXpert
   }
   
 
-  MzLabInputOligomerTreeViewDlg *
+  void
+  MzLabWnd::help()
+  {
+    // Explain to the user what's the point of having matches
+    // performed.
+
+    QString msg(tr("To perform a match operation one must have\n"
+                   "two lists of (m/z,z) pairs already available.\n\n"
+                   "One list must be selected in the Catalogue 1\n"
+                   "and the other list must be selected in the Catalogue 2.\n\n"
+                   "There are two kinds of \"match work\" operations : \n"
+                   "One that aims at finding matches between (m/z,z) pairs\n"
+                   "in each of the two lists and one that aims at finding\n"
+                   "(m/z,z) pairs that have no match in the other list.\n\n"
+                   "How are match (or non-match) works performed ?\n"
+                   "One match work is performed by searching in\n"
+                   "the first list (Catalogue 1) all the (m/z,z) pairs \n"
+                   "that are also in the second list (Catalogue 2).\n\n"
+                   "One non-match work is performed by searching in the\n"
+                   "first list (Catalogue 1) all the (m/z,z) pairs that are\n"
+                   "not found in the second list (Catalogue 2).\n"));
+    
+    QMessageBox::warning(this,
+                         tr("massXpert: mz Lab"),
+                         msg,
+                         QMessageBox::Ok);
+  }
+  
+
+  MzLabInputOligomerTableViewDlg *
   MzLabWnd::findDlg(const QString &name)
   {
     for (int iter = 0; iter < m_dlgList.size(); ++iter)
       {
-	MzLabInputOligomerTreeViewDlg *dlg = m_dlgList.at(iter);
+	MzLabInputOligomerTableViewDlg *dlg = m_dlgList.at(iter);
 	
 	if(dlg->name() == name)
 	  return dlg;
@@ -926,7 +1039,7 @@ namespace massXpert
   {
     QString text = item->text();
     
-    MzLabInputOligomerTreeViewDlg *dlg = findDlg(text);
+    MzLabInputOligomerTableViewDlg *dlg = findDlg(text);
     Q_ASSERT(dlg);
     
     dlg->show();
@@ -961,7 +1074,7 @@ namespace massXpert
   void
   MzLabWnd::destroyDlg(const QString &name)
   {
-    MzLabInputOligomerTreeViewDlg *dlg = findDlg(name);
+    MzLabInputOligomerTableViewDlg *dlg = findDlg(name);
     Q_ASSERT(dlg);
 
     // Remove the dialog window from the list of dialog windows and
