@@ -680,6 +680,30 @@ namespace massXpert
     decimalPlacesOptionsAct->setStatusTip(tr("Decimal places"));
     connect(decimalPlacesOptionsAct, SIGNAL(triggered()), this, 
 	     SLOT(decimalPlacesOptions()));
+
+    // Calculator (preseed with whole sequence masses)
+    newCalculatorWholeSequenceMassesAct = 
+      new QAction(tr("New calculator (whole seq. masses)"), this);
+
+    newCalculatorWholeSequenceMassesAct->setStatusTip
+      (tr("Start new calculator preseeded with whole sequence masses"));
+    
+    connect(newCalculatorWholeSequenceMassesAct,
+            SIGNAL(triggered()), 
+            this, 
+            SLOT(newCalculatorWholeSequenceMasses()));
+
+    // Calculator (preseed with selected sequence masses)
+    newCalculatorSelectedSequenceMassesAct = 
+      new QAction(tr("New calculator (selected seq. masses)"), this);
+    
+    newCalculatorSelectedSequenceMassesAct->setStatusTip
+      (tr("Start new calculator preseeded with selected sequence masses"));
+    
+    connect(newCalculatorSelectedSequenceMassesAct,
+            SIGNAL(triggered()), 
+            this, 
+            SLOT(newCalculatorSelectedSequenceMasses()));
   }
 
 
@@ -723,6 +747,10 @@ namespace massXpert
 
     optionsMenu = menuBar()->addMenu(tr("&Options"));
     optionsMenu->addAction(decimalPlacesOptionsAct);
+
+    calculatorMenu = menuBar()->addMenu(tr("&Calculator"));
+    calculatorMenu->addAction(newCalculatorWholeSequenceMassesAct);
+    calculatorMenu->addAction(newCalculatorSelectedSequenceMassesAct);
   }
 
 
@@ -1871,6 +1899,58 @@ namespace massXpert
   }
 
 
+  void 
+  SequenceEditorWnd::wholeSequenceMasses(double *mono, double *avg)
+  {
+    QString mass;
+    bool ok = false;
+    Application *application = static_cast<Application *>(qApp);
+    
+    // First get the text string and make a double out of it.
+
+    if(mono)
+      {
+        mass = m_ui.monoWholeMassLineEdit->text();
+        *mono = application->locale().toDouble(mass, &ok);
+      }
+
+    if(avg)
+      {
+        mass = m_ui.avgWholeMassLineEdit->text();
+        *avg = application->locale().toDouble(mass, &ok);
+      }
+    
+
+  }
+  
+  
+
+  void 
+  SequenceEditorWnd::selectedSequenceMasses(double *mono, double *avg)
+  {
+    QString mass;
+    bool ok = false;
+    Application *application = static_cast<Application *>(qApp);
+   
+    if(mono)
+      {
+        mass = m_ui.monoSelectionMassLineEdit->text();
+        *mono = application->locale().toDouble(mass, &ok);
+      }
+    
+    if(avg)
+      {
+        mass = m_ui.avgSelectionMassLineEdit->text();
+        *avg = application->locale().toDouble(mass, &ok);
+      }
+    
+
+  }
+  
+
+
+
+
   bool 
   SequenceEditorWnd::maybeSave()
   {
@@ -2735,10 +2815,36 @@ namespace massXpert
   void
   SequenceEditorWnd::mzCalculation()
   {
+    // We should try to feed the mz calculation dialog with the mass
+    // data from the sequence editor window. If there is a real
+    // selection, than use the masses of the selected
+    // region(s). Otherwise use the masses for the whole sequence.
+
+    CoordinateList coordList;
+    double mono = 0;
+    double avg = 0;
+        
+    if (mpa_editorGraphicsView->selectionIndices(&coordList))
+      {
+        // There is real selection, set the masses for that selection
+        // in the newly created dialog window.
+
+        selectedSequenceMasses(&mono, &avg);
+      }
+    else
+      {
+        wholeSequenceMasses(&mono, &avg);
+        qDebug() << __FILE__ << __LINE__;
+      }
+
+    qDebug() << __FILE__ << __LINE__
+             << "mono:" << mono << "avg:" << avg;
+    
+    
     MzCalculationDlg *dlg = 
       new MzCalculationDlg(this, 
 			    mpa_polymer->polChemDef(),
-			    &m_ionizeRule);
+                           &m_ionizeRule, mono, avg);
   
     dlg->show();
   }
@@ -2822,6 +2928,55 @@ namespace massXpert
   }
   
 
+  void
+  SequenceEditorWnd::newCalculatorWholeSequenceMasses()
+  {
+    // We want to start a new calculator preseeded with the whole
+    // sequence mono/avg masses. Let's get them right away.
+
+    QString mono = m_ui.monoWholeMassLineEdit->text();
+    QString avg = m_ui.avgWholeMassLineEdit->text();
+
+    newCalculator(mono, avg);
+  }
+  
+
+  void
+  SequenceEditorWnd::newCalculatorSelectedSequenceMasses()
+  {
+    // We want to start a new calculator preseeded with the selected
+    // sequence mono/avg masses. Let's get them right away.
+
+    QString mono = m_ui.monoSelectionMassLineEdit->text();
+    QString avg = m_ui.avgSelectionMassLineEdit->text();
+
+    newCalculator(mono, avg);
+  }
+
+
+  void
+  SequenceEditorWnd::newCalculator(QString mono, QString avg)
+  {
+    // To start a new calculator, we have to get the name of the file
+    // that contains the polymer chemistry definition. Let's get that
+    // filePath from the polymer chemistry definition of *this*
+    // sequence editor window.
+
+    QString filePath = mp_polChemDef->filePath();
+    
+    // Open a calculator window with the proper polymer chemistry
+    // definition and also with the proper masses to preseed.
+
+    CalculatorWnd *calculatorWnd = new CalculatorWnd(filePath, mono, avg);
+
+    // At this time we have a calculator window that is fully
+    // initialized and functional. We can add its pointer to the list
+    // of such windows that is stored in the application object.
+    Application *application = static_cast<Application *>(qApp);
+    application->calculatorWndList()->append(calculatorWnd);
+  }    
+
+
   void 
   SequenceEditorWnd::crossLinksPartiallyEncompassedSlot(int count)
   {
