@@ -592,9 +592,10 @@ namespace massXpert
  
       	CleaveOligomer *oligomer = 
 	  new CleaveOligomer(mp_polymer, 
-			      name, m_cleaveOptions.name(),
-			      Ponderable(),
-			      leftIndex, rightIndex, partialCleavageValue);
+                             name, m_cleaveOptions.name(),
+                             Ponderable(),
+                             leftIndex, rightIndex, 
+                             partialCleavageValue, m_calcOptions);
 	
 	partialOligomerList.append(oligomer);
 
@@ -628,10 +629,11 @@ namespace massXpert
   
     CleaveOligomer *oligomer = 
       new CleaveOligomer(mp_polymer, 
-			  name, m_cleaveOptions.name(),
-			  Ponderable(),
-			  leftIndex, rightIndex, partialCleavageValue);
-  
+                         name, m_cleaveOptions.name(),
+                         Ponderable(),
+                         leftIndex, rightIndex, 
+                         partialCleavageValue, m_calcOptions);
+    
     partialOligomerList.append(oligomer);
   
     // At this point all the skeleton oligomers have been computed for
@@ -1130,9 +1132,9 @@ namespace massXpert
   void 
   Cleaver::emptyOligomerList()
   {
-    while (!mp_oligomerList->isEmpty())
+    while (mp_oligomerList->size())
       {
-	delete mp_oligomerList->first();
+	delete mp_oligomerList->takeFirst();
       }
   }
 
