@@ -446,7 +446,8 @@ namespace massXpert
     bool shouldBeFormula = false;
     bool wasMinusSign = false;
 	
-    Q_ASSERT(refList.size());
+    if(refList.isEmpty())
+      qFatal("Fatal error at %s@%d. Aborting.",__FILE__, __LINE__);
   
     // We are asked to put all the '+' components of the formula
     // into corresponding formula and the same for the '-' components.
@@ -638,7 +639,6 @@ namespace massXpert
     AtomCount *atomCount = 0;
 
     bool wasDigit = false;
-    bool wasUpper = false;
     bool gotUpper = false;
 
     Q_ASSERT(refList.size());
@@ -686,7 +686,6 @@ namespace massXpert
 	    parsedCount.append(curChar);
 
 	    wasDigit = true;
-	    wasUpper = false;
 
 	    continue;
 	  }
@@ -704,7 +703,6 @@ namespace massXpert
 
 	    // Let the people know that we have parsed a lowercase char
 	    // and not a digit.
-	    wasUpper = false;
 	    wasDigit = false;
 	  }
 	else if (curChar.category() == QChar::Letter_Uppercase)
@@ -719,7 +717,6 @@ namespace massXpert
 		parsedSymbol.append(curChar);
 
 		gotUpper = true;
-		wasUpper = true;
 		wasDigit = false;
 		continue;
 	      }
@@ -757,7 +754,7 @@ namespace massXpert
 		    // an error in the conversion ?
 
 		    delete atomCount;
-
+		    
 		    return false;
 		  }
 	      
@@ -791,7 +788,6 @@ namespace massXpert
 	  
 	    wasDigit = false;
 	    gotUpper = true;
-	    wasUpper = true;
 	  }
 	// end(curChar.category() == QChar::Letter_Uppercase)
       }
@@ -935,7 +931,7 @@ namespace massXpert
   }
 
 
-  //! Checks the syntax of the \p formula.
+//! Checks the syntax of the \p formula.
   /*! The syntax of the \p formula is checked by verifying that the
     letters and ciphers in the formula are correctly placed. That is, we
     want that the ciphers appear after an atom symbol and not before
@@ -957,9 +953,7 @@ namespace massXpert
   {
     QChar curChar;
 
-    bool wasDigit = false;
     bool gotUpper = false;
-    bool wasUpper = false;
     bool wasSign = false;
 
     // Because the formula that we are analyzing might contain a title
@@ -995,8 +989,6 @@ namespace massXpert
 	      return false;
 	  
 	    wasSign = false;
-	    wasDigit = true;
-	    wasUpper = false;
 
 	    continue;
 	  }
@@ -1017,8 +1009,6 @@ namespace massXpert
 	    // Let the people know that we have parsed a lowercase char
 	    // and not a digit.
 	    wasSign = false;
-	    wasUpper = false;
-	    wasDigit = false;
 	  }
 	else if (curChar.category() == QChar::Letter_Uppercase)
 	  {
@@ -1028,9 +1018,7 @@ namespace massXpert
 	    // Let the people know what we got:
 	  
 	    wasSign = false;
-	    wasDigit = false;
 	    gotUpper = true;
-	    wasUpper = true;
 	  }
 	else 
 	  {
@@ -1044,9 +1032,7 @@ namespace massXpert
 	      }
 	  
 	    wasSign = true;
-	    wasDigit = false;
 	    gotUpper = false;
-	    wasUpper = false;
 	  }
       }
     // end for (int iter = 0 ; iter < localFormula.length() ; ++iter)
@@ -1083,17 +1069,6 @@ namespace massXpert
   }
 
 
-  QString 
-  Formula::simplify()
-  {
-    // We have +CH3 -C +C2H6O4N5P
-    // We want to have the result of this: that is C2H8O4N5P
-    
-    return elementalComposition();
-  }
-  
-
-
   //! Validates the formula.
   /*! The validation of the formula involves:
 
@@ -1256,73 +1231,155 @@ namespace massXpert
   QString
   Formula::elementalComposition() const
   {
-    QString composition;
-    bool nextNegative = false;
+    // We have a list of AtomCount objects which might have either a
+    // positive or a negative count.
+
+    // We want to provide a formula that lists the positive component
+    // first and the negative component last.
+
+    // Each positive/negative component will list the atoms in the
+    // conventional order : CxxHxxNxxOxx and all the rest in
+    // alphabetical order.
+
+    // For each atomCount in the m_atomCountList list, prepare a
+    // string like "C12" and "H13", for example, in two distinct
+    // QStringList's, one for positive count values and one for
+    // negative count values.
+
+    QStringList negativeStringList;
+    QStringList positiveStringList;
     
     for (int iter = 0; iter < m_atomCountList.size(); ++iter)
       {
-	AtomCount *atomCount = 0;
-	
-	AtomCount *nextAtomCount = 0;
-	
-	if (iter + 1 < m_atomCountList.size())
-	  {
-	    nextAtomCount = m_atomCountList.at(iter + 1);
-	    
-	    if (nextAtomCount->count() > 0)
-	      nextNegative = false;
-	    else
-	      nextNegative = true;
-	  }
-	else
-	  {
-	    // We are already at the last item,
-	    // so we do not add a space at the end of a negative
-	    // item:
-	
-	    nextNegative = true;
-	  }
-	
-	atomCount = m_atomCountList.at(iter);
-
-	// Do not list item if it is 0-count.
-	if (!atomCount->count())
-	  continue;
-	
-	if (atomCount->count() > 0)
-	  {
-	    // The count is positive, just list the item as "C12"
-
-	  composition += QString("%1%2")
-	    .arg(atomCount->symbol())
-	    .arg(atomCount->count());
-
-	  nextNegative = false;
-	  }
-	else
-	  {
-	    // The count is negative, so we have to list the item as
-	    // "-C12", but then, we do not want to have "-C-12"
-	    // because count will be negative, thus we change its
-	    // value to positive by *ying by -1. 
-
-	    // Note how we put a %3 arg after the item, so that if
-	    // next items are positive they do not get factorized
-	    // erroneously under the negative sign. This only in case
-	    // the next item is positive. If the previous item was
-	    // negative, we do not need any supplementary space
-	    // because we are adding a '-' anyways.
-
-	    composition += QString("-%1%2%3")
-	      .arg(atomCount->symbol())
-	      .arg(atomCount->count() * -1)
-	      .arg(nextNegative ? "" : "+");
-
-	    nextNegative = true;
-	  }
-      }
+	AtomCount *atomCount = m_atomCountList.at(iter);
         
-    return composition;
+        if(atomCount->count() < 0)
+          {
+            negativeStringList.append(QString("%1%2")
+                                      .arg(atomCount->symbol())
+                                      .arg(-1 * atomCount->count()));
+          }
+        else
+          {  
+            positiveStringList.append(QString("%1%2")
+                                      .arg(atomCount->symbol())
+                                      .arg(atomCount->count()));
+          }
+      }
+    
+    // Sort the lists.
+
+    negativeStringList.sort();
+    positiveStringList.sort();
+
+    // We want to provide for each positive and negative components of
+    // the initial formula object, an elemental formula that complies
+    // with the convention : first the C atom, next the H, N, O atoms
+    // and all the subsequent ones in alphabetical order. 
+
+    // Thus we look for the four C, H, N, O atoms, and we create the
+    // initial part of the elemental formula. Each time we find one
+    // such atom we remove it from the list, so that we can later just
+    // append all the remaining atoms, since we have sorted the lists
+    // above.
+
+    // The positive component
+    // ======================
+
+    int atomIndex = 0;
+    QString positiveComponentString;
+    
+    // Carbon
+    atomIndex = positiveStringList.indexOf(QRegExp("C\\d+"));
+    if(atomIndex != -1)
+      {
+        positiveComponentString += positiveStringList.at(atomIndex);
+        positiveStringList.removeAt(atomIndex);
+      }
+  
+    // Hydrogen
+    atomIndex = positiveStringList.indexOf(QRegExp("H\\d+"));
+    if(atomIndex != -1)
+      {
+        positiveComponentString += positiveStringList.at(atomIndex);
+        positiveStringList.removeAt(atomIndex);
+      }
+  
+    // Nitrogen
+    atomIndex = positiveStringList.indexOf(QRegExp("N\\d+"));
+    if(atomIndex != -1)
+      {
+        positiveComponentString += positiveStringList.at(atomIndex);
+        positiveStringList.removeAt(atomIndex);
+      }
+  
+    // Oxygen
+    atomIndex = positiveStringList.indexOf(QRegExp("O\\d+"));
+    if(atomIndex != -1)
+      {
+        positiveComponentString += positiveStringList.at(atomIndex);
+        positiveStringList.removeAt(atomIndex);
+      }
+
+    // All the other items are already sorted and can be appended.
+
+    positiveComponentString += positiveStringList.join("");
+
+
+    // The negative component
+    // ======================
+
+    atomIndex = 0;
+    QString negativeComponentString;
+    
+    // Carbon
+    atomIndex = negativeStringList.indexOf(QRegExp("C\\d+"));
+    if(atomIndex != -1)
+      {
+        negativeComponentString += negativeStringList.at(atomIndex);
+        negativeStringList.removeAt(atomIndex);
+      }
+  
+    // Hydrogen
+    atomIndex = negativeStringList.indexOf(QRegExp("H\\d+"));
+    if(atomIndex != -1)
+      {
+        negativeComponentString += negativeStringList.at(atomIndex);
+        negativeStringList.removeAt(atomIndex);
+      }
+  
+    // Nitrogen
+    atomIndex = negativeStringList.indexOf(QRegExp("N\\d+"));
+    if(atomIndex != -1)
+      {
+        negativeComponentString += negativeStringList.at(atomIndex);
+        negativeStringList.removeAt(atomIndex);
+      }
+  
+    // Oxygen
+    atomIndex = negativeStringList.indexOf(QRegExp("O\\d+"));
+    if(atomIndex != -1)
+      {
+        negativeComponentString += negativeStringList.at(atomIndex);
+        negativeStringList.removeAt(atomIndex);
+      }
+
+    // All the other items are already sorted and can be appended.
+
+    negativeComponentString += negativeStringList.join("");
+
+    // Create the final elemental formula that comprises both the
+    // positive and negative element. First the positive element and
+    // then the negative one. Only append the negative one, prepended
+    // with '-' if the string is non-empty.
+
+    QString elementalComposition = positiveComponentString;
+
+    if (!negativeComponentString.isEmpty())
+      elementalComposition += QString("-%1")
+      .arg(negativeComponentString);
+
+    return elementalComposition;
   }
 
 
@@ -1452,14 +1509,10 @@ namespace massXpert
 
     for (int iter = 0; iter < m_atomCountList.size(); ++iter)
       {
-	AtomCount *atomCount = 0;
-      
-	atomCount = m_atomCountList.at(iter);
+	AtomCount *atomCount = m_atomCountList.at(iter);
 
-	Atom listAtom;
-      
 	if(Atom::isSymbolInList(atomCount->symbol(), 
-				  refList, atomCount) == -1)
+                                refList, atomCount) == -1)
 	  return false;
       }
 
