@@ -70,11 +70,14 @@ namespace massXpert
     ionized.
   */
   Oligomer::Oligomer(Polymer *polymer , const QString &name, 
-		      const Ponderable &ponderable,
-		      const IonizeRule &ionizeRule, bool isIonized,
-		      int startIndex, int endIndex)
+                     const Ponderable &ponderable,
+                     const IonizeRule &ionizeRule,
+                     const CalcOptions & calcOptions,
+                     bool isIonized,
+                     int startIndex, int endIndex)
     : Ionizable(polymer->polChemDef(), name, ponderable, 
-		 ionizeRule, isIonized), mp_polymer(polymer)
+                ionizeRule, isIonized), mp_polymer(polymer),
+      m_calcOptions(calcOptions)
   {
     setStartEndIndices(startIndex, endIndex);
   }
@@ -110,15 +113,20 @@ namespace massXpert
     ionized.
   */
   Oligomer::Oligomer(const PolChemDef *polChemDef, const QString &name, 
-		      const Ponderable &ponderable,
-		      const IonizeRule &ionizeRule, bool isIonized,
-		      int startIndex, int endIndex)
+                     const Ponderable &ponderable,
+                     const IonizeRule &ionizeRule,
+                     const CalcOptions &calcOptions,
+                     bool isIonized,
+                     int startIndex, int endIndex)
     : Ionizable(polChemDef, name, ponderable, ionizeRule, isIonized), 
-      mp_polymer(0)
+      mp_polymer(0), m_calcOptions(calcOptions)
   {
     setStartEndIndices(startIndex, endIndex);
+    if (startIndex < 0)
+      qDebug() << __FILE__ << __LINE__
+               << "Construct with startIndex:" << startIndex;
   }
-
+  
 
   //! Constructs an oligomer.
   /*!
@@ -141,12 +149,16 @@ namespace massXpert
     Coordinates item in the CoordinateList).
   */
   Oligomer::Oligomer(Polymer *polymer, const QString &name, 
-		      const Ponderable &ponderable,
-		      int startIndex, int endIndex)
+                     const Ponderable &ponderable,
+                     int startIndex, int endIndex,
+                     const CalcOptions &calcOptions)
     : Ionizable(polymer->polChemDef(), name, ponderable), 
-      mp_polymer(polymer)
+      mp_polymer(polymer), m_calcOptions(calcOptions)
   {
     setStartEndIndices(startIndex, endIndex);
+    if (startIndex < 0)
+      qDebug() << __FILE__ << __LINE__
+               << "Construct with startIndex:" << startIndex;
   }
 
 
@@ -175,11 +187,16 @@ namespace massXpert
     Coordinates item in the CoordinateList).
   */
   Oligomer::Oligomer(const PolChemDef *polChemDef, const QString &name, 
-		      const Ponderable &ponderable,
-		      int startIndex, int endIndex)
-    : Ionizable(polChemDef, name, ponderable), mp_polymer(0)
+                     const Ponderable &ponderable,
+                     const CalcOptions &calcOptions,
+                     int startIndex, int endIndex)
+    : Ionizable(polChemDef, name, ponderable), mp_polymer(0), 
+      m_calcOptions(calcOptions)
   {
     setStartEndIndices(startIndex, endIndex);
+    if (startIndex < 0)
+      qDebug() << __FILE__ << __LINE__
+               << "Construct with startIndex:" << startIndex;
   }
 
 
@@ -201,13 +218,17 @@ namespace massXpert
 
     \param avg Average mass.
   */
-  Oligomer::Oligomer(Polymer *polymer , const QString &name, 
-		      double mono, double avg,
-		      int startIndex, int endIndex)
+  Oligomer::Oligomer(Polymer *polymer , const QString &name,
+                     double mono, double avg,
+                     int startIndex, int endIndex, 
+                     const CalcOptions &calcOptions)
     : Ionizable(mp_polymer->polChemDef(), name, Ponderable(mono, avg)),
-      mp_polymer(polymer)
+      mp_polymer(polymer), m_calcOptions(calcOptions)
   {
     setStartEndIndices(startIndex, endIndex);
+    if (startIndex < 0)
+      qDebug() << __FILE__ << __LINE__
+               << "Construct with startIndex:" << startIndex;
   }
 
 
@@ -234,10 +255,11 @@ namespace massXpert
     \param avg Average mass.
   */
   Oligomer::Oligomer(const PolChemDef *polChemDef, const QString &name, 
-		      double mono, double avg,
-		      int startIndex, int endIndex)
+                     const CalcOptions &calcOptions,
+                     double mono, double avg,
+                     int startIndex, int endIndex)
     : Ionizable(polChemDef, name, Ponderable(mono, avg)),
-      mp_polymer(0)
+      mp_polymer(0),m_calcOptions(calcOptions)
   {
     setStartEndIndices(startIndex, endIndex);
   }
@@ -250,8 +272,8 @@ namespace massXpert
     \param other oligomer to be used as a mold.
   */
   Oligomer::Oligomer(const Oligomer &other)
-    : CoordinateList(other), Ionizable(other),
-      mp_polymer(other.mp_polymer)
+    : CoordinateList(other), Ionizable(other), PropListHolder(other),
+      mp_polymer(other.mp_polymer), m_calcOptions(other.m_calcOptions)
   {
   }
   
@@ -307,6 +329,10 @@ namespace massXpert
   void 
   Oligomer::setStartIndex(int value)
   {
+    if (value < 0)
+      qDebug() << __FILE__ << __LINE__
+               << "setStartIndex with value:" << value;
+
     if (!CoordinateList::size())
       {
 	Coordinates *coordinates = new Coordinates();
@@ -404,6 +430,19 @@ namespace massXpert
     return m_ionizeRule;
   }
 
+  void
+  Oligomer::setCalcOptions(const CalcOptions &calcOptions)
+  {
+    m_calcOptions = calcOptions;
+  }
+  
+
+  const CalcOptions &
+  Oligomer::calcOptions() const
+  {
+    return m_calcOptions;
+  }
+  
 
   const Monomer &
   Oligomer::atLeftEnd() const
@@ -708,13 +747,13 @@ namespace massXpert
       {
 	Coordinates *coordinates = CoordinateList::at(iter);
 
-	for(int iter = coordinates->start(); 
-	     iter < coordinates->end() + 1; ++iter)
+	for(int jter = coordinates->start(); 
+	     jter < coordinates->end() + 1; ++jter)
 	  {
 // 	    qDebug() << __FILE__ << __LINE__ << "Going to call at() with value"
 // 		      << iter;
 	    
-	    const Monomer *monomer = mp_polymer->at(iter);
+	    const Monomer *monomer = mp_polymer->at(jter);
 	    
 	    if (monomer->isModified())
 	      return true;
