@@ -924,19 +924,36 @@ namespace massXpert
   void *
   NoDeletePointerProp::clone() const
   {
-    qDebug() << __FILE__ << __LINE__ << "Does nothing";
-    Q_ASSERT(0);
+    NoDeletePointerProp *prop = 0;
     
-    return 0;
+    if (mpa_data)
+      prop = 
+        new NoDeletePointerProp(m_name, 
+                                static_cast<void *>(mpa_data));
+
+    return static_cast<void *>(prop);
   }
   
   
   void 
   NoDeletePointerProp::cloneOut(void *other) const
   {
-    qDebug() << __FILE__ << __LINE__ << "Does nothing";
-    Q_ASSERT(0);
- }
+    Q_ASSERT(other);
+    
+    NoDeletePointerProp *otherProp = static_cast<NoDeletePointerProp *>(other);
+    
+    if (otherProp->mpa_data)
+      otherProp->deleteData();
+    
+    otherProp->m_name = m_name;
+    
+    if (mpa_data)
+      {
+	otherProp->mpa_data = static_cast<void *>(mpa_data);
+      }
+    else
+      otherProp->mpa_data = 0;  
+  }
   
   
   void 
